/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2013.02.02.
*   Chip         -  Atmel ATmega8
*   Compiler     -  avr-gcc (WinAVR )
*
*   PC AT (PS2) billentyuzet kezelese.
*   A prg. a scan kodokat a soros portra irja ki.
*
********************************************************************************
*   AVR PB0 14 - LED - 470R - +5V
*
*   AVR INT0 4 - KBD CLOCK 5 
*   AVR PD3  5 - KBD DATA  1
*                KBD +VCC  4 - +5V
*                KBD GND   3 - GND
*
*******************************************************************************/

#define F_CPU          4000000                   // orajel 4MHz
#define BAUDRATE       9600
#define MYUBRR         F_CPU/ 16UL/ BAUDRATE - 1




#include "tkiraaly_atmega8.h"




#define LED                  0
#define LED_ENABLE           BS( DDRB, LED)
#define LED_BE               BC( PORTB, LED)
#define LED_KI               BS( PORTB, LED)




#define KBD_DATA             BTS( PIND, 3)       // billentyuzetrol bejovo adat bit

// MCUCR beallitasai
#define INT0_0               0                   // IT 0 szintnel
#define INT0_1               0B00000001          // IT 1 szintnel
#define INT0_LEFUTO          0B00000010          // IT 1->0 elre
#define INT0_FELFUTO         0B00000011          // IT 0->1 elre
#define INT0_ENABLE          GICR|= 0B01000000   // INT0 engedelyezese
                                                 
#define INT1_0               0                   // IT 0 szintnel
#define INT1_1               0B00000100          // IT 1 szintnel
#define INT1_LEFUTO          0B00001000          // IT 1->0 elre
#define INT1_FELFUTO         0B00001100          // IT 0->1 elre
#define INT1_ENABLE          GICR|= 0B10000000   // INT1 engedelyezese




UC volatile kbd= 0;                              // vett billentyu scan kodja
UC kbd_prev= 0;                                  // elozo billentyu scan kodja
UC kbd_input= 0;                                 // bitek -> kod
UC kbd_count= 11;                                // bit szamlalo




void usart_putc( UC);                            // USART egy karakter kuldese
void usart_hex( UC);                             // USART 1 byte kuldese hexadecimalisan
UC num2hexc( UC);                                // szam hexadecimalis szamjeggye alakitasa
void kbd_decoder( void);




ISR( INT0_vect)                                  // INT0 megszakitas kezelese
{
   if( kbd_count < 11 && kbd_count > 2)          // csak 3..10 bit az adat, tobbi eldobasa
   {
      kbd_input=( kbd_input >> 1);
      if( KBD_DATA) kbd_input|= 0B10000000;      // '1' eltarolasa
   }
   if( --kbd_count == 0)                         // osszes bit bejott
   {
      if( ( kbd_input != 0xE0) &&                // 0xE0 elnyomas
          ( kbd_input != 0xF0) &&                // 0xF0 es ismetles elnyomas
          ( kbd_prev  != 0xF0)) kbd= kbd_input; 
      kbd_prev= kbd_input; 
      kbd_count= 11;
   }
}




int main( void)
{
   UBRRL= MYUBRR;                                // USART inicializalasa
   UBRRH= ( MYUBRR)>> 8;
   UCSRC= SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;
   UCSRB= TX_ENABLE;

   MCUCR= INT0_LEFUTO;                           // billentyuzet kezeleshez
   INT0_ENABLE;

   IT_ENABLE;
  
   LED_ENABLE;
   LED_BE;

   for(;;)
   {
      if( kbd)                                   // ha nyomtak le billentyut
      {
         usart_hex( kbd);
         kbd= 0;
         usart_putc( ' ');
      }
   }
}




void usart_putc( UC c)                           // USART egy karakter kuldese
{
    while( BTC( UCSRA, UDRE));                   // 1, ha kesz az adat fogadasara
    UDR= c;
}




void usart_hex( UC c)                            // USART 1 byte kuldese hexadecimalisan
{
   usart_putc( num2hexc(c >> 4));
   usart_putc( num2hexc(c));
}




UC num2hexc( UC c)                               // szam hexadecimalis szamjeggye alakitasa
{
   c&= 0x0F;
   if( c < 10) c+= '0';
   else        c+= 'A'- 10;
   return c;
}
